@@ -18,10 +18,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include "config.h"
@@ -183,13 +179,16 @@ static int handle_ts1_read(struct osmo_fd *bfd)
 		handle_dahdi_exception(e1i_ts);
 	else if (ret < 0) {
 		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		msgb_free(msg);
+		return ret;
+	}
+	if (ret <= 3) {
+		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		msgb_free(msg);
 		return ret;
 	}
 	msgb_put(msg, ret - 2);
-	if (ret <= 3) {
-		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
-		return ret;
-	}
+	msg->l2h = msg->data;
 
 	return e1inp_rx_ts_lapd(e1i_ts, msg);
 }
@@ -281,8 +280,10 @@ static void handle_hdlc_write(struct osmo_fd *bfd)
 
 	/* get the next msg for this timeslot */
 	msg = e1inp_tx_ts(e1i_ts, NULL);
-	if (!msg)
+	if (!msg) {
+		osmo_fd_write_disable(bfd);
 		return;
+	}
 
 	if (msgb_tailroom(msg) >= 2) {
 		/* two bytes of space for the FCS added by DAHDI in the kernel */
@@ -317,13 +318,16 @@ static int handle_hdlc_read(struct osmo_fd *bfd)
 		handle_dahdi_exception(e1i_ts);
 	else if (ret < 0) {
 		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		msgb_free(msg);
+		return ret;
+	}
+	if (ret <= 3) {
+		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
+		msgb_free(msg);
 		return ret;
 	}
 	msgb_put(msg, ret - 2);
-	if (ret <= 3) {
-		LOGPITS(e1i_ts, DLMI, LOGL_ERROR, "%s read failed %d (%s)\n", __func__, ret, strerror(errno));
-		return ret;
-	}
+	msg->l2h = msg->data;
 
 	return e1inp_rx_ts(e1i_ts, msg, 0, 0);
 }
@@ -377,6 +381,7 @@ static int handle_tsX_read(struct osmo_fd *bfd)
 	ret = read(bfd->fd, msg->data, D_TSX_ALLOC_SIZE);
 	if (ret < 0 || ret != D_TSX_ALLOC_SIZE) {
 		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "read error  %d %s\n", ret, strerror(errno));
+		msgb_free(msg);
 		return ret;
 	}
 
@@ -406,8 +411,10 @@ static int handle_ts_raw_write(struct osmo_fd *bfd)
 
 	/* get the next msg for this timeslot */
 	msg = e1inp_tx_ts(e1i_ts, NULL);
-	if (!msg)
+	if (!msg) {
+		osmo_fd_write_disable(bfd);
 		return 0;
+	}
 
 	if (msg->len != D_BCHAN_TX_GRAN) {
 		/* This might lead to a transmit underrun, as we call tx
@@ -444,6 +451,7 @@ static int handle_ts_raw_read(struct osmo_fd *bfd)
 	ret = read(bfd->fd, msg->data, D_TSX_ALLOC_SIZE);
 	if (ret < 0 || ret != D_TSX_ALLOC_SIZE) {
 		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "read error  %d %s\n", ret, strerror(errno));
+		msgb_free(msg);
 		return ret;
 	}
 
